/// <reference path="../References.d.ts"/>
import * as React from 'react';
import * as Blueprint from '@blueprintjs/core';

interface Props {
	title: string;
	content: string;
	examples?: string[];
}

interface State {
	popover: boolean;
}

const css = {
	box: {
		position: 'relative',
		display: 'inline',
	} as React.CSSProperties,
	content: {
		padding: '20px',
	} as React.CSSProperties,
	button: {
		position: 'absolute',
		top: '-7px',
		left: '-2px',
		padding: '7px',
		background: 'none',
		opacity: 0.3,
	} as React.CSSProperties,
	popover: {
		width: '230px',
	} as React.CSSProperties,
	popoverTarget: {
		top: '9px',
		left: '18px',
	} as React.CSSProperties,
	dialog: {
		maxWidth: '400px',
		margin: '30px 20px',
	} as React.CSSProperties,
	inputFirst: {
		width: '100%',
		marginTop: '15px',
	} as React.CSSProperties,
	input: {
		width: '100%',
		marginTop: '5px',
	} as React.CSSProperties,
};

let dialog = true;

export default class Help extends React.Component<Props, State> {
	constructor(props: Props, context: any) {
		super(props, context);
		this.state = {
			popover: false,
		};
	}

	autoSelect = (evt: React.MouseEvent<HTMLInputElement>): void => {
		evt.currentTarget.select();
	}

	render(): JSX.Element {
		let examplesElem: JSX.Element[] = [];
		for (let i = 0; i < (this.props.examples || []).length; i++) {
			examplesElem.push(
				<input
					key={'example-' + i}
					className="bp3-input"
					style={i === 0 ? css.inputFirst : css.input}
					disabled={false}
					readOnly={true}
					type="text"
					autoCapitalize="off"
					spellCheck={false}
					value={this.props.examples[i]}
					onClick={this.autoSelect}
				/>
			);
		}

		let helpElm: JSX.Element;
		if (this.state.popover) {
			if (dialog) {
				helpElm = <Blueprint.Dialog
					title={this.props.title}
					style={css.dialog}
					isOpen={this.state.popover}
					usePortal={true}
					portalContainer={document.body}
					onClose={(): void => {
						this.setState({
							...this.state,
							popover: false,
						});
					}}
				>
					<div className="bp3-dialog-body">
						{this.props.content}
						{examplesElem}
					</div>
					<div className="bp3-dialog-footer">
						<div className="bp3-dialog-footer-actions">
							<button
								className="bp3-button"
								type="button"
								onClick={(): void => {
									this.setState({
										...this.state,
										popover: !this.state.popover,
									});
								}}
							>Close</button>
						</div>
					</div>
				</Blueprint.Dialog>;
			} else {
				helpElm = <span
					className="bp3-popover-target"
					style={css.popoverTarget}
				>
				<span className="bp3-overlay bp3-overlay-inline">
					<span>
						<div
							className={'bp3-transition-container ' +
							'bp3-tether-element-attached-middle ' +
							'bp3-tether-element-attached-left ' +
							'bp3-tether-target-attached-middle ' +
							'bp3-tether-target-attached-right bp3-overlay-content'}
							style={css.popover}
						>
							<div className="bp3-popover">
								<div className="bp3-popover-arrow">
									<svg viewBox="0 0 30 30">
										<path
											className="bp3-popover-arrow-border"
											d={'M8.11 6.302c1.015-.936 1.887-2.922 ' +
											'1.887-4.297v26c0-1.378-' +
											'.868-3.357-1.888-4.297L.925 ' +
											'17.09c-1.237-1.14-1.233-3.034 0-4.17L8.11 6.302z'}
										/>
										<path
											className="bp3-popover-arrow-fill"
											d={'M8.787 7.036c1.22-1.125 2.21-3.376 ' +
											'2.21-5.03V0v30-2.005c0-1.654-' +
											'.983-3.9-2.21-5.03l-7.183-6.616c-' +
											'.81-.746-.802-1.96 0-2.7l7.183-6.614z'}
										/>
									</svg>
								</div>
								<div
									className="bp3-popover-content"
									style={css.content}
								>
									<h5>{this.props.title}</h5>
									<div>
										{this.props.content}
										{examplesElem}
									</div>
								</div>
							</div>
						</div>
					</span>
				</span>
			</span>;
			}
		}

		return <div style={css.box}>
			<div
				className="bp3-button bp3-minimal bp3-icon-help"
				style={css.button}
				onClick={(): void => {
					this.setState({
						...this.state,
						popover: !this.state.popover,
					});
				}}
			/>
			{helpElm}
		</div>;
	}
}
